#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {$ARGC < 4 || [flagLoc $ARGV -help]} \
   {
    puts stderr "Display NMRPipe Format Series as Text Data;"
    puts stderr " $argv0 fileName1 \[fileName2 ...\]"
    puts stderr "Options:"
    puts stderr " -index indexType \[Pts\]   Point Index Used for Output."
    puts stderr " -real                    Display X-Axis Real Data Only."
    puts stderr " -imag                    Display X-Axis Imaginary Data Only."
    puts stderr "Notes:"
    puts stderr " 1. All data files in the series are assumed to have"
    puts stderr "    the same sizes and spectral windows."
    puts stderr " 2. Point Index Keywords: Pts Hz PPM % None"
    exit 0
   }

set realFlag 1
set imagFlag 1
set mode     pts
set verbose  0

getArgD $ARGV -index mode

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -real]} {set imagFlag 0}
if {[flagLoc $ARGV -imag]} {set realFlag 0}

#
#--------------------------------------------------------------------------

set aSize [llength $argv]
set zSize 0
set iz    0

for {set ia 0} {$ia < $aSize} {incr ia } \
   {
    set thisArg [lindex $argv $ia]

    if {![strcasecmp $thisArg "-real"]}   continue
    if {![strcasecmp $thisArg "-imag"]}   continue
    if {![strcasecmp $thisArg "-verb"]}   continue
    if {![strcasecmp $thisArg "-noverb"]} continue
    if {![strcasecmp $thisArg "-notk"]}   continue
    if {![strcasecmp $thisArg "--"]}      continue

    if {![strcasecmp $thisArg "-index"]} \
       {
        incr ia
        continue
       }
 
    set inName $thisArg

    if {![file exists $inName]} \
       {
        puts stderr "Error Finding Input File $inName"
        exit 1
       }

    incr zSize
   }

for {set ia 0} {$ia < $aSize} {incr ia } \
   {
    set thisArg [lindex $argv $ia]

    if {![strcasecmp $thisArg "-real"]} continue
    if {![strcasecmp $thisArg "-imag"]} continue
    if {![strcasecmp $thisArg "-notk"]} continue
    if {![strcasecmp $thisArg "--"]}    continue
 
    if {![strcasecmp $thisArg "-index"]} \
       {
        incr ia
        continue
       }

    set  inName $thisArg
    incr iz

    set inUnit [sysOpen $inName r]
    set error  [rdFDATAU $inUnit fdata]

    if {$error} {exit 1}

    set qState [getParmI $fdata FDQUADFLAG NULL_DIM]

    if {$qState == 0.0} \
       {
        set qSize 2
       } \
    else \
       {
        set qSize 1
       }

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set xSize    [getParmI $fdata NDSIZE CUR_XDIM]
    set ySize    [getParmI $fdata NDSIZE CUR_YDIM]

    if {$dimCount < 2} {set ySize 1}

    set n        [expr $xSize*$ySize*$qSize]

    set rdata [sysMalloc $n float]

    sysRead  $inUnit $rdata $n
    sysClose $inUnit

    if {$verbose} {puts stderr $inName}

    set i 0

    for {set iy 1} {$iy <= $ySize} {incr iy} \
        {
         for {set ix 1} {$ix <= $xSize} {incr ix} \
            {
             if {$qSize == 2} \
                {
                 set rVal   [getPtrVal $rdata $i]
                 set iVal   [getPtrVal $rdata [expr $i + $xSize]]

                 if {!$realFlag} \
                    {
                     set valStr [format " % 14.5f" $iVal]
                    } \
                 elseif {!$imagFlag} \
                    {
                     set valStr [format " % 14.5f" $rVal]
                    } \
                 else \
                    {
                     set valStr [format " % 14.5f % 14.5f" $rVal $iVal]
                    }
                } \
             else \
                {
                 set rVal   [getPtrVal $rdata $i]
                 set valStr [format " % 14.5f" $rVal]
                }

             if {![strcasecmp $mode None]} \
                {
                 set locStr ""
                } \
             elseif {![strcasecmp $mode pts]} \
                {
                 if {$ySize > 1 && $zSize > 1} \
                    {
                     set locStr [format " %5d %5d %4d" $ix $iy $iz]
                    } \
                 elseif {$ySize == 1 && $zSize > 1} \
                    {
                     set locStr [format " %5d %4d" $ix $iz]
                    } \
                 elseif {$ySize > 1 && $zSize == 1} \
                    {
                     set locStr [format " %5d %5d" $ix $iy]
                    } \
                 else \
                    {
                     set locStr [format " %5d" $ix]
                    }
                } \
             else \
                {
                 set px 0.0
                 set py 0.0
                 set pz 0.0

                 if {$dimCount >= 1} {set px [pnt2spec $fdata X_AXIS $ix $mode]}
                 if {$dimCount >= 2} {set py [pnt2spec $fdata Y_AXIS $iy $mode]}
                 if {$dimCount >= 3} {set pz [pnt2spec $fdata Z_AXIS $iz $mode]}

                 switch -exact [string tolower $mode] \
                    {
                     %       {set fmt "% 8.3f"}
                     ppm     {set fmt "% 8.3f"}
                     hz      {set fmt "% 10.3f"}
                     default {set fmt "% e"}
                    }

                 if {$ySize > 1 && $zSize > 1} \
                    {
                     set locStr [format " $fmt $fmt $fmt" $px $py $pz]
                    } \
                 elseif {$ySize == 1 && $zSize > 1} \
                    {
                     set locStr [format " $fmt %4d" $px $iz]
                    } \
                 elseif {$ySize > 1 && $zSize == 1} \
                    {
                     set locStr [format " $fmt $fmt" $px $py]
                    } \
                 else \
                    {
                     set locStr [format " $fmt" $px]
                    }
                }

             puts " $locStr  $valStr"
             incr i
            }
        }

    sysFree  $fdata
    sysFree  $rdata
   }

exit 0
